/******************************************************
Library for 24cXX from bitbanding STM32F030
		by Sergi 28.05.2017
******************************************************/
#ifndef __i2c_soft
#define __i2c_soft

// #define TYPE_24C32

// Connect i2c_SDA to PORTB7, i2c_SCL to PORTB6

#define SCL_H GPIOB->BSRR = GPIO_BSRR_BS_6
#define SCL_L GPIOB->BSRR = GPIO_BSRR_BR_6
#define SDA_H GPIOB->BSRR = GPIO_BSRR_BS_7
#define SDA_L GPIOB->BSRR = GPIO_BSRR_BR_7
#define SDA_IN GPIOB->MODER &=~  GPIO_MODER_MODER7
#define SDA_OUT GPIOB->MODER |= GPIO_MODER_MODER7_0
#define SDA_STAT (GPIOB->IDR & 0x80)
#define i2c_timeset 2

#ifndef __delay_us
#define __delay_us
void delay_us(volatile unsigned long nTime)
	{ 
 nTime=nTime<<2;
 while(nTime != 0)
 {nTime--;
	nTime++;
	nTime--;	
 }
 }; 
#endif
	
unsigned char reserv[32];

int i2c_send_start (void)
{
	//   SDA
	SCL_H;			   		// SCL  1
	SDA_IN;		//SDA  
	if (!SDA_STAT) return 1;
	SDA_OUT;		//SDA  
	SDA_L;				   	// SDA  0
	delay_us(i2c_timeset);
	SCL_L;				   	// SCL  0
	delay_us(i2c_timeset);
	return 0;
};

void i2c_send_stop (void)
{
	SDA_OUT;		//SDA  
	SCL_L;			   		// SCL  0
	SDA_L;			  	 	// SDA  0
	delay_us(i2c_timeset);
	SCL_H;			  	 	// SCL  1
	delay_us(i2c_timeset);
	SDA_H;			   		// SDA  1
	delay_us(i2c_timeset);
};

void i2c_send_wr (unsigned char wr)
{
	unsigned char tmp = 0x80;
	while (tmp)
	{
		SDA_OUT;				//SDA  
		delay_us(i2c_timeset);
		if (tmp & wr)SDA_H;		// SDA  1
		else SDA_L;			   	// SDA  0
		tmp = tmp>>1;
		delay_us(i2c_timeset);
		SCL_H;			  			 	// SCL  1
		delay_us(i2c_timeset);
		SCL_L;			   				// SCL  0
	};
	SDA_IN; 					//SDA  
	delay_us(i2c_timeset);
	SCL_H;			 				  	// SCL  1
	delay_us(i2c_timeset);
	SCL_L;			  				 	// SCL  0
	delay_us(i2c_timeset);
};

unsigned char i2c_send_rd (void)
{
	unsigned char tmp = 0x80,ret = 0;
	SDA_IN;		 //SDA  
	while (tmp)
	{
		SCL_H;			 		// SCL  1
		delay_us(i2c_timeset);
		if (SDA_STAT) ret |= tmp;
		tmp = tmp>>1;
		delay_us(i2c_timeset);
		SCL_L;			   	// SCL  0
		delay_us(i2c_timeset);
	};
	SCL_H;			  	 	// SCL  1
	delay_us(i2c_timeset);
	SCL_L;			   		// SCL  0
	delay_us(i2c_timeset);														//NAK!
	return ret;
};

unsigned char i2c_read_data (int adrr)
{
	int tmp = 0;
	i2c_send_start();
	i2c_send_wr (0xA0);
#ifdef TYPE_24C32 
	i2c_send_wr (adrr >> 8);  //for 24c32
#endif
	i2c_send_wr (adrr & 0xff);
	i2c_send_start();
	i2c_send_wr (0xA1);
	tmp = i2c_send_rd();
	i2c_send_stop();
	return tmp;
};

void i2c_write_data(int adrr,unsigned char data)
	{
	i2c_send_start();
	i2c_send_wr (0xA0);
#ifdef TYPE_24C32 
	i2c_send_wr (adrr >> 8);  //for 24c32
#endif
	i2c_send_wr (adrr & 0xff);
	i2c_send_wr (data);
	i2c_send_stop();
	delay_us(10000);		
	};

void i2c_write_32(int adrr,long int data)
{
	i2c_write_data(adrr,(data >> 24));
	i2c_write_data(adrr + 1,((data >> 16) & 0xff));
	i2c_write_data(adrr + 2,((data >> 8) & 0xff));
	i2c_write_data(adrr + 3,(data & 0xff));
};

long int i2c_read_32(int adrr)
{
	long int tp = 0;
	tp = tp | ((long int)(i2c_read_data(adrr)) << 24);
	tp = tp | ((long int)(i2c_read_data(adrr + 1)) << 16);
	tp = tp | ((long int)(i2c_read_data(adrr + 2)) << 8);
	tp = tp | i2c_read_data(adrr + 3);
	return tp;
};

void save_page (int adrr)
{
	unsigned char i;
	i2c_send_start();
	i2c_send_wr (0xA0);
#ifdef TYPE_24C32 
	i2c_send_wr (adrr >> 8);  //for 24c32
#endif
	i2c_send_wr (adrr & 0xff);
	for (i=0;i<32;i++)
	{
		i2c_send_wr (reserv[i]);
	};
	i2c_send_stop();
	delay_us(10000);	
};

void load_page (int adrr)
{
	unsigned char i;
	for (i=0;i<32;i++)
	{
	reserv[i] = i2c_read_data(i+adrr);
	};
};

#endif
